--
-- Quantum
-- This is the specialization for Ladewagen
--
-- M@D Author  Headshot XXL
-- M@D date  14.05.2009
--
-- > Copyright (C) Headshot XXL - www.planet-ls.de < --

Quantum = {};

function Quantum.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function Quantum:load(xmlFile)

    local forageWgnSound = getXMLString(xmlFile, "vehicle.forageWgnSound#file");
    if forageWgnSound ~= nil and forageWgnSound ~= "" then
        forageWgnSound = Utils.getFilename(forageWgnSound, self.baseDirectory);
        self.forageWgnSound = createSample("forageWgnSound");
        self.forageWgnSoundEnabled = false;
        loadSample(self.forageWgnSound, forageWgnSound, false);
        self.forageWgnSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#pitchOffset"), 1);
        self.forageWgnSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#volume"), 1.0);
    end;
	
		local pickupNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.pickup#index"));
    if pickupNode ~= nil then
        self.pickup = {};
        self.pickup.node = pickupNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.pickup#minRot"));
        self.pickup.minRot = {};
        self.pickup.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.pickup.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.pickup.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.pickup#maxRot"));
        self.pickup.maxRot = {};
        self.pickup.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.pickup.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.pickup.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.pickup.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.pickup#rotTime"), 2)*1000;
        self.pickup.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.pickup#touchRotLimit"), 10));
    end;
	
	self.drum1= Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.drum#index1"));
    self.drumRotationScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.drum#rotationScale"), 1);

    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
    self.wasToFast = false;
    self.isTurnedOn = false;
	self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
	self.pickupMax = false;
end;

function Quantum:delete()
    if self.forageWgnSound ~= nil then
        delete(self.forageWgnSound);
    end;
end;

function Quantum:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Quantum:keyEvent(unicode, sym, modifier, isDown)
end;

function Quantum:update(dt)

    if self:getIsActiveForInput() then
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
            self.isTurnedOn = not self.isTurnedOn;
        end;
    end;
	
	if self.pickup ~= nil then
        local x, y, z = getRotation(self.pickup.node);
        local rot = {x,y,z};
        local newRot = Utils.getMovedLimitedValues(rot, self.pickup.maxRot, self.pickup.minRot, 3, self.pickup.rotTime, dt, not self.pickupMax);
        setRotation(self.pickup.node, unpack(newRot));
    end;
	
	if self.drum1 ~= nil and self.isTurnedOn then
		rotate(self.drum1, self.drumRotationScale * 100, 0, 0); 
	end;

    self.wasToFast = false;
    if self:getIsActive() then
        if self.isTurnedOn and self.fillLevel <= self.capacity then
		
			self.pickupMax = true;
			
            local toFast = self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed*3600 > 29;
            if not toFast then				
				local area =0;
                for k, cuttingArea in pairs(self.cuttingAreas) do
                    local x,y,z = getWorldTranslation(cuttingArea.start);
                    local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
                    local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

                    for fruitType,v in pairs(self.fillTypes) do
                        if fruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
							if self.currentFruitType == fruitType or self.currentFruitType == FruitUtil.FRUITTYPE_UNKNOWN then
								local area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0)*g_currentMission.windrowCutLongRatio;
								area = area + Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0);
								if area > 0 then
									self.currentFruitType = fruitType;
								end;
								local pixelToQm = 2048 / 4096 * 2048 / 4096  / g_currentMission.maxFruitValue; -- 4096px are mapped to 2048m
								local literPerQm = FruitUtil.fruitIndexToDesc[fruitType].literPerQm * (1 + 0.5 * (3 - g_currentMission.missionStats.difficulty));
								local qm = area*pixelToQm;
								local deltaLevel = qm*literPerQm * self.fillScale;
								self:setFillLevel(self.fillLevel+deltaLevel, self.currentFruitType);
							end;
                        end;
                    end;
                end;	
				if self.fillLevel == 0 then
					self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
				end;			
            end;

            self.wasToFast = toFast;
		
			self.allowFillFromAir = false;
			
            if not self.forageWgnSoundEnabled and self:getIsActiveForSound() then
                playSample(self.forageWgnSound, 0, self.forageWgnSoundVolume, 0);
                setSamplePitch(self.forageWgnSound, self.forageWgnSoundPitchOffset);
                self.forageWgnSoundEnabled = true;
            end;
		else
			self.allowFillFromAir = true;
			self.pickupMax = false;
        end;

        if self.forageWgnSoundEnabled and not self.isTurnedOn then
            stopSample(self.forageWgnSound);
            self.forageWgnSoundEnabled = false;
        end;

    end;

end;

function Quantum:draw()
    if self.wasToFast then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;
    if self.isTurnedOn then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    end;
end;

function Quantum:onDetach()
    if self.deactivateOnDetach then
        Quantum.onDeactivate(self);
    else
        Quantum.onDeactivateSounds(self);
    end;
end;

function Quantum:onLeave()
    if self.deactivateOnLeave then
        Quantum.onDeactivate(self);
    else
        Quantum.onDeactivateSounds(self);
    end;
end;

function Quantum:onDeactivate()
    self.isTurnedOn = false;
    Quantum.onDeactivateSounds(self);
end;

function Quantum:onDeactivateSounds()
    if self.forageWgnSoundEnabled then
        stopSample(self.forageWgnSound);
        self.forageWgnSoundEnabled = false;
    end;
end;
